﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//// Copyright(c) HNS Co., Ltd. All rights reserved
//// HNS Site 주소 : https://www.hnsts.co.kr
//// SmartX New Framework Reference Guide : https://www.smartx.co.kr
/////////////////////////////////////////////////// [저작권] ///////////////////////////////////////
//// 본 자료(소스 및 이미지)의 저작권은 ㈜에이치앤에스에 있습니다. 
//// 본 자료는 개발하시는데 참고자료 용도로 사용하실 수 있으며 저작권 표시 없이 복사,게재, 
//// 출판 하실 수 없습니다. 외부 게재 사용 시 반드시 출처 항목에 회사명과 사이트 주소를 
//// 반드시 명시해 주시기 바랍니다.
//// [표기 예] 출처 : ㈜에이치앤에스(www.hnsts.co.kr) 또는 ㈜HNS(www.hnsts.co.kr)
//// 단, 프로그램을 개발할 때는 참고용으로 사용하셔도 됩니다.
//////////////////////////////////////////// [수정 및 업데이트] ////////////////////////////////////
//// [중요] 소스코드는 참고 및 학습용으로 버그 및 여러가지 문제가 있을 수 있습니다.
//// 제품 개발 시 내용과 예제 코드는 수정 및 검증작업을 직접 하셔야 하며 문제 발생에 
//// 대한 책임은 ㈜HNS사와 무관합니다.
//// 본사는 자료를 최신내용으로 유지하기 위해 노력하고 있으며 제공되는 정보의 오류 및 
//// 개선이 필요할 경우 사전 공지 없이 업데이트 될 수 있습니다. 
//// 본 자료(소스 및 이미지)의 문제점 발견 시 본사로 문의주시면 검토하여 자료를 수정하도록 
//// 하겠습니다.
//// 최신 수정일 : 2023.12.15
////////////////////////////////////////////////// [문의방법] //////////////////////////////////////
//// 1. 전화 : 02-6402-8001 (H/W : 2번, S/W : 3번)
//// 2. E-Mail : app@smartx.co.kr
//// 3. QnA : https://www.hnsts.co.kr/Community/QnA
/////////////////////////////////////////////// [프로그램 설명] ////////////////////////////////////
//// 프로그램 이름 : CNetworkDeviceManagement Class_SmartTCPMultiServer
//// 프로그램 설명 : CNetworkDeviceManagement Class를 SmartTCPMultiServer에 적용하여 유무선 네트워크 
//// 연결 지연에 따른 처리 및 유선 네트워크 사용시 MAC Address 설정 기능을 사용할 수 있는  TCP통신의
//// Server 측 예제 프로그램입니다.

//// 사용된 SmartX Framework Version : 5.0.0.10
////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace TechNote80_Server_CS
{
    class CNetworkDeviceManagement
    {
        public enum NetDeviceType : int
        {
            WiredLAN = 0,
            WirelessLAN_RT2501 = 1,
            WirelessLAN_RT2870 = 2,
        }

        public enum FailStatus : int
        {
            IP_Allocation_Failed = 0,
            ServerConnect_Failed = 1,
        }

        public struct NetIPSetInfos
        {
            public NetIPSetInfos(int DHCP, string sDeviceIP, string sGateWay, string sSubNetMask, string sPrimaryDNS, string sSecondaryDNS, string sPrimaryWINS, string sSecondaryWINS)
            {
                iDHCP = DHCP;
                DeviceIP = sDeviceIP;
                GateWay = sGateWay;
                SubNetMask = sSubNetMask;
                PrimaryDNS = sPrimaryDNS;
                SecondaryDNS = sSecondaryDNS;
                PrimaryWINS = sPrimaryWINS;
                SecondaryWINS = sSecondaryWINS;
            }
            public int iDHCP;
            public string DeviceIP;
            public string GateWay;
            public string SubNetMask;
            public string PrimaryDNS;
            public string SecondaryDNS;
            public string PrimaryWINS;
            public string SecondaryWINS;
        };

        private SmartX.SmartConfigs m_pSmartConfig;
        private NetDeviceType m_NetworkDeviceType;
        private string m_strFailMessage;
        private int m_iIPAllocationRetryMax;
        private int m_iIPAllocationRetryCount;
        private int m_iServerConnectRetryMax;
        private int m_iServerConnectRetryCount;
        private string m_strAPAddress;
        private string m_strAPName;
        private string m_strAPPassword;
        private NetIPSetInfos m_IPInfos;
        private System.Windows.Forms.Timer m_CompletionTimer;
        private System.Windows.Forms.Timer m_MainTimer;
        private string m_strMACAddressFilePath;

        public delegate void FailHandler(FailStatus fStatus);
        public event FailHandler On_Failed;
        public delegate void CompletionHandler(ref bool bServerConnect);
        public event CompletionHandler On_Completion;

        public CNetworkDeviceManagement()
        {
            m_pSmartConfig = null;
            m_NetworkDeviceType = NetDeviceType.WiredLAN;
            m_strFailMessage = "";
            m_iIPAllocationRetryMax = 20;
            m_iIPAllocationRetryCount = 0;
            m_iServerConnectRetryMax = 10;
            m_iServerConnectRetryCount = 0;
            m_strAPAddress = "";
            m_strAPName = "";
            m_strAPPassword = "";
            m_CompletionTimer = new System.Windows.Forms.Timer();
            m_CompletionTimer.Tick += new EventHandler(CompletionTimer_Tick);
            m_MainTimer = new System.Windows.Forms.Timer();
            m_MainTimer.Tick += new EventHandler(MainTimer_Tick);
        }

        public SmartX.SmartConfigs pSmartConfig
        {
            set
            {
                m_pSmartConfig = value;
            }
        }

        public NetDeviceType NetworkDeviceType
        {
            set
            {
                m_NetworkDeviceType = value;
            }
        }

        public string FailedMessage
        {
            set
            {
                m_strFailMessage = value;
            }
        }
        
        public int IPAllocationRetryMax
        {
            set
            {
                m_iIPAllocationRetryMax = value;
            }
        }

        public int GetIPAllocationRetryCnt
        {
            get
            {
                return m_iIPAllocationRetryCount;
            }
        }

        public int GetServerConnectRetryCnt
        {
            get
            {
                return m_iServerConnectRetryCount;
            }
        }

        public int ServerConnectRetryMax
        {
            set
            {
                m_iServerConnectRetryMax = value;
            }
        }

        public void SetIPSettings(NetIPSetInfos IPInfos)
        {
            m_IPInfos = IPInfos;
        }

        public void SetAPInfos(string strAPAddress, string strAPName, string strAPPassword)
        {
            m_strAPAddress = strAPAddress;
            m_strAPName = strAPName;
            m_strAPPassword = strAPPassword;
        }

        public string MACAddressFilePathName
        {
            set
            {
                m_strMACAddressFilePath = value;
            }
        }

        public void Start()
        {
            IPInfosChange(m_IPInfos);
            m_MainTimer.Interval = 700;
            m_MainTimer.Enabled = true;
        }

        public void Stop()
        {
            m_MainTimer.Enabled = false;
            m_CompletionTimer.Enabled = false;
            m_iIPAllocationRetryCount = 0;
            m_iServerConnectRetryCount = 0;
        }

        // 유무선 네트워크 타입에 따른 IP 할당 시도 및 할당 여부에 따라 IP 할당 재시도와 Server 연결을 진행 하는 함수
        private bool Network_Connect()
        {
            // Server 연결을 진행하기 위한 TImer Interval
            m_CompletionTimer.Interval = 300;

            // 유선,무선 랜 타입에 따른 IP 할당을 시도
            switch (m_NetworkDeviceType)
            {                    
                // 유선랜 사용
                case NetDeviceType.WiredLAN:
                    // 유선랜으로 네트워크에 연결 했을때 IP를 할당 받아 네트워크 통신이 가능한지 확인 
                    if (m_pSmartConfig.IPSettings.IsLocalNetworkAvailable() == true)
                    {
                        m_CompletionTimer.Enabled = true;
                        return true;
                    }
                    break;
                // 구형 무선랜 사용
                case NetDeviceType.WirelessLAN_RT2501:
                    m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501;
                    if (m_strAPPassword == "")
                    {
                        // 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                        if (m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501, m_strAPName, m_strAPAddress, 1, 1, 10) == SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED)
                        {
                            m_CompletionTimer.Enabled = true;
                            return true;
                        }
                    }
                    else
                    {
                        // 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                        if (m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501, m_strAPName, m_strAPPassword, m_strAPAddress, 1, 1, 10) == SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED)
                        {
                            m_CompletionTimer.Enabled = true;
                            return true;
                        }
                    }
                    break;
                // 신형 무선랜 사용
                case NetDeviceType.WirelessLAN_RT2870:
                    m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870;
                    if (m_strAPPassword == "")
                    {
                        // 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                        if (m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870, m_strAPName, m_strAPAddress,1, 1, 10) == SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED)
                        {
                            m_CompletionTimer.Enabled = true;
                            return true;
                        }
                    }
                    else
                    {
                        // 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                        if (m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870, m_strAPName, m_strAPPassword, m_strAPAddress, 1, 1, 10) == SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED)
                        {
                            m_CompletionTimer.Enabled = true;
                            return true;
                        }
                    }
                    break;
            }
            // IP 할당 시도 최대 횟수까지 재시도
            if (++m_iIPAllocationRetryCount < m_iIPAllocationRetryMax)
            {
                // IP 할당을 진행하기 위한 Timer 시작
                m_MainTimer.Enabled = true;
            }
            else
            {
                // IP 할당 실패에 따른 처리
                if (m_strFailMessage != "")
                {
                    SmartX.SmartMessageBox.Size(515, 190);
                    SmartX.SmartMessageBox.Show(m_strFailMessage, "IP 할당 실패", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Hand, System.Windows.Forms.MessageBoxDefaultButton.Button1);
                }
                if (On_Failed != null)
                {
                    On_Failed(FailStatus.IP_Allocation_Failed);
                }
            }

            return false;
        }

        // 유무선 네트워크 타입에 따라 IP, GateWay, SubNetMask, DNS, WINS 주소를 설정하는 함수
        private void IPInfosChange(NetIPSetInfos netIPInfos)
        {
            bool bUpdateFlag = false;

            switch (m_NetworkDeviceType)
            {
                // 유선랜 사용
                case NetDeviceType.WiredLAN:
                    // 동적 IP 사용
                    if (m_pSmartConfig.IPSettings.DHCPEnable != netIPInfos.iDHCP)
                    {
                        m_pSmartConfig.IPSettings.DHCPEnable = netIPInfos.iDHCP;
                    }
                    // 고정 IP 사용
                    if (netIPInfos.iDHCP == 0)
                    {
                        if ((netIPInfos.DeviceIP != null) && (m_pSmartConfig.IPSettings.DeviceIP != netIPInfos.DeviceIP))
                        {
                            m_pSmartConfig.IPSettings.DeviceIP = netIPInfos.DeviceIP;
                        }
                        if ((netIPInfos.GateWay != null) && (m_pSmartConfig.IPSettings.GateWay != netIPInfos.GateWay))
                        {
                            m_pSmartConfig.IPSettings.GateWay = netIPInfos.GateWay;
                        }
                        if ((netIPInfos.SubNetMask != null) && (m_pSmartConfig.IPSettings.SubNetMask != netIPInfos.SubNetMask))
                        {
                            m_pSmartConfig.IPSettings.SubNetMask = netIPInfos.SubNetMask;
                        }
                        if ((netIPInfos.PrimaryDNS != null) && (m_pSmartConfig.IPSettings.PrimaryDNS != netIPInfos.PrimaryDNS))
                        {
                            m_pSmartConfig.IPSettings.PrimaryDNS = netIPInfos.PrimaryDNS;
                        }
                        if ((netIPInfos.SecondaryDNS != null) && (m_pSmartConfig.IPSettings.SecondaryDNS != netIPInfos.SecondaryDNS))
                        {
                            m_pSmartConfig.IPSettings.SecondaryDNS = netIPInfos.SecondaryDNS;
                        }
                        if ((netIPInfos.PrimaryWINS != null) && (m_pSmartConfig.IPSettings.PrimaryWINS != netIPInfos.PrimaryWINS))
                        {
                            m_pSmartConfig.IPSettings.PrimaryWINS = netIPInfos.PrimaryWINS;
                        }
                        if ((netIPInfos.SecondaryWINS != null) && (m_pSmartConfig.IPSettings.SecondaryWINS != netIPInfos.SecondaryWINS))
                        {
                            m_pSmartConfig.IPSettings.SecondaryWINS = netIPInfos.SecondaryWINS;
                        }
                    }
                    // IP 주소 설정값을 레지스트리에 저장
                    m_pSmartConfig.IPSettings.Save();
                    // 설정값을 적용
                    m_pSmartConfig.IPSettings.SetApply();
                    break;

                // 무선랜 사용
                case NetDeviceType.WirelessLAN_RT2501:
                case NetDeviceType.WirelessLAN_RT2870:
                    if (m_NetworkDeviceType == NetDeviceType.WirelessLAN_RT2501)
                    {
                        m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501;
                    }
                    else
                    {
                        m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870;
                    }
                    // 동적 IP 사용
                    if (m_pSmartConfig.USBWirelessIPSettings.DHCPEnable != netIPInfos.iDHCP)
                    {
                        m_pSmartConfig.USBWirelessIPSettings.DHCPEnable = netIPInfos.iDHCP;
                        bUpdateFlag = true;
                    }
                    // 고정 IP 사용
                    if (netIPInfos.iDHCP == 0)
                    {
                        if ((netIPInfos.DeviceIP != null) && (m_pSmartConfig.USBWirelessIPSettings.DeviceIP != netIPInfos.DeviceIP))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.DeviceIP = netIPInfos.DeviceIP;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.GateWay != null) && (m_pSmartConfig.USBWirelessIPSettings.GateWay != netIPInfos.GateWay))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.GateWay = netIPInfos.GateWay;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.SubNetMask != null) && (m_pSmartConfig.USBWirelessIPSettings.SubNetMask != netIPInfos.SubNetMask))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.SubNetMask = netIPInfos.SubNetMask;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.PrimaryDNS != null) && (m_pSmartConfig.USBWirelessIPSettings.PrimaryDNS != netIPInfos.PrimaryDNS))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.PrimaryDNS = netIPInfos.PrimaryDNS;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.SecondaryDNS != null) && (m_pSmartConfig.USBWirelessIPSettings.SecondaryDNS != netIPInfos.SecondaryDNS))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.SecondaryDNS = netIPInfos.SecondaryDNS;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.PrimaryWINS != null) && (m_pSmartConfig.USBWirelessIPSettings.PrimaryWINS != netIPInfos.PrimaryWINS))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.PrimaryWINS = netIPInfos.PrimaryWINS;
                            bUpdateFlag = true;
                        }
                        if ((netIPInfos.SecondaryWINS != null) && (m_pSmartConfig.USBWirelessIPSettings.SecondaryWINS != netIPInfos.SecondaryWINS))
                        {
                            m_pSmartConfig.USBWirelessIPSettings.SecondaryWINS = netIPInfos.SecondaryWINS;
                            bUpdateFlag = true;
                        }
                    }

                    if (bUpdateFlag == true)
                    {
                        // IP 주소 설정값을 레지스트리에 저장
                        m_pSmartConfig.USBWirelessIPSettings.Save();
                        // 설정값을 적용
                        m_pSmartConfig.USBWirelessIPSettings.SetApply();
                    }
                    break;
            }
        }

        // MAC Adress 설정을 위한 함수
        public bool MacAddress_Setting()
        {
            SmartX.SmartFile smartFile1 = new SmartX.SmartFile();

            // txt 파일에서 읽어온 MAC Address를 저장할 변수
            string strRawMacaddress;
            // MAC Address를 ':' 로 분할하여 각각 나눈 값을 저장할 배열
            string[] strMacAddress;
            // MAC Address 설정을 위해 분할된 값을 하나의 MAC Address로 저장할 배열
            byte[] bSetMacAddress = new byte[6];  

            // MAC Address 설정 값을 가지고 있는 txt 파일이 존재 하지 않는경우
            if ((m_strMACAddressFilePath == "") || (System.IO.File.Exists(m_strMACAddressFilePath) == false))
            {
                return false;
            }

            // 읽어올 MAC Address txt 파일의 경로 설정 및 File Open
            smartFile1.FilePathName = m_strMACAddressFilePath;
            smartFile1.Open();

            // txt 파일의 MAC Address 설정 값을 문자열로 읽기
            strRawMacaddress = smartFile1.StringType.Read();            
            smartFile1.Close();

            // MAC Address 값의 길이가 17 자리가 아닌 경우
            if (strRawMacaddress.Length != 17)
            {
                return false;
            }

            strMacAddress = strRawMacaddress.Split(':');

            // 분할하여 나눈 MAC Address 값의 길이가 6 자리가 아닌 경우
            if (strMacAddress.Length != 6)
            {
                return false;
            }

            try
            {
                // 분할하여 나눈 MAC Address 길이 만큼 반복
                for (int i = 0; i < strMacAddress.Length; i++)
                {
                    bSetMacAddress[i] = (byte)Convert.ToInt16(strMacAddress[i].ToString(), 16);
                }
            }
            catch (FormatException)
            {
                return false;
            }

            byte[] bMacAddress = m_pSmartConfig.IPSettings.GetMACAddress();

            // 설정 되어 있는 MAC Address와 파일로 읽어온 MAC Address 값이 같지 않은 경우
            if (Enumerable.SequenceEqual(bSetMacAddress, bMacAddress) == false)
            {
                // 파일에서 읽어온 MAC Address값으로 설정
                m_pSmartConfig.IPSettings.SetMACAddress(bSetMacAddress);

                // MAC Address 적용을 위한 IEC-Series 재부팅 여부 확인
                DialogResult eRebootResult = SmartX.SmartMessageBox.Show("MAC Address Setting이 완료되었습니다.\n재부팅을 진행하시겠습니까?", "재부팅 여부 확인", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (eRebootResult == DialogResult.Yes)
                {
                    SmartX.SmartConfigs.ReBoot();
                }
            }
            return true;
        }

        // IP 할당을 시도하기 위한 이벤트
        private void MainTimer_Tick(object sender, EventArgs e)
        {
            m_MainTimer.Enabled = false;

            Network_Connect();
        }

        // IP 할당 여부에 따라 Server 연결을 시도하기 위한 이벤트
        private void CompletionTimer_Tick(object sender, EventArgs e)
        {
            m_CompletionTimer.Enabled = false;

            // Server 연결 성공, 실패 여부를 확인하기 위한 변수
            bool bServerConnectFlag = false;
            
            if (On_Completion != null)
            {
                // Server 연결에 성공, 실패 여부 확인
                On_Completion(ref bServerConnectFlag);

                //// Server 연결에 실패한 경우
                if (bServerConnectFlag == false)
                {
                    // Server 연결 시도 최대 횟수 까지 재시도
                    if (++m_iServerConnectRetryCount <= m_iServerConnectRetryMax)
                    {
                        m_CompletionTimer.Enabled = true;
                    }
                    else
                    {

                        if (On_Failed != null)
                        {
                            // Server 연결에 실패한 경우
                            On_Failed(FailStatus.ServerConnect_Failed);
                        }
                    }
                }
            }
        }
    }
}