﻿'////////////////////////////////////////////////////////////////////////////////////////////////////
'//// Copyright(c) HNS Co., Ltd. All rights reserved
'//// HNS Site 주소 : https://www.hnsts.co.kr
'//// SmartX New Framework Reference Guide : https://www.smartx.co.kr
'/////////////////////////////////////////////////// [저작권] ///////////////////////////////////////
'//// 본 자료(소스 및 이미지)의 저작권은 ㈜에이치앤에스에 있습니다. 
'//// 본 자료는 개발하시는데 참고자료 용도로 사용하실 수 있으며 저작권 표시 없이 복사,게재, 
'//// 출판 하실 수 없습니다. 외부 게재 사용 시 반드시 출처 항목에 회사명과 사이트 주소를 
'//// 반드시 명시해 주시기 바랍니다.
'//// [표기 예] 출처 : ㈜에이치앤에스(www.hnsts.co.kr) 또는 ㈜HNS(www.hnsts.co.kr)
'//// 단, 프로그램을 개발할 때는 참고용으로 사용하셔도 됩니다.
'//////////////////////////////////////////// [수정 및 업데이트] ////////////////////////////////////
'//// [중요] 소스코드는 참고 및 학습용으로 버그 및 여러가지 문제가 있을 수 있습니다.
'//// 제품 개발 시 내용과 예제 코드는 수정 및 검증작업을 직접 하셔야 하며 문제 발생에 
'//// 대한 책임은 ㈜HNS사와 무관합니다.
'//// 본사는 자료를 최신내용으로 유지하기 위해 노력하고 있으며 제공되는 정보의 오류 및 
'//// 개선이 필요할 경우 사전 공지 없이 업데이트 될 수 있습니다. 
'//// 본 자료(소스 및 이미지)의 문제점 발견 시 본사로 문의주시면 검토하여 자료를 수정하도록 
'//// 하겠습니다.
'//// 최신 수정일 : 2023.12.15
'////////////////////////////////////////////////// [문의방법] //////////////////////////////////////
'//// 1. 전화 : 02-6402-8001 (H/W : 2번, S/W : 3번)
'//// 2. E-Mail : app@smartx.co.kr
'//// 3. QnA : https://www.hnsts.co.kr/Community/QnA
'/////////////////////////////////////////////// [프로그램 설명] ////////////////////////////////////
'//// 프로그램 이름 : CNetworkDeviceManagement Class_SmartTCPMultiServer
'//// 프로그램 설명 : CNetworkDeviceManagement Class를 SmartTCPMultiServer에 적용하여 유무선 네트워크  
'//// 연결 지연에 따른 처리 및 유선 네트워크 사용시 MAC Address 설정 기능을 사용할 수 있는  TCP통신의
'//// Server 측 예제 프로그램입니다.

'//// 사용된 SmartX Framework Version : 5.0.0.10
'////////////////////////////////////////////////////////////////////////////////////////////////////

Public Class CNetworkDeviceManagement

    Public Enum NetDeviceType As Integer
        WiredLAN = 0
        WirelessLAN_RT2501 = 1
        WirelessLAN_RT2870 = 2
    End Enum

    Public Enum FailStatus As Integer
        IP_Allocation_Failed = 0
        ServerConnect_Failed = 1
    End Enum

    Public Structure NetIPSetInfos
        Public Sub New(ByVal DHCP As Integer, ByVal sDeviceIP As String, ByVal sGateWay As String, ByVal sSubNetMask As String, ByVal sPrimaryDNS As String, ByVal sSecondaryDNS As String, ByVal sPrimaryWINS As String, ByVal sSecondaryWINS As String)
            iDHCP = DHCP
            DeviceIP = sDeviceIP
            GateWay = sGateWay
            SubNetMask = sSubNetMask
            PrimaryDNS = sPrimaryDNS
            SecondaryDNS = sSecondaryDNS
            PrimaryWINS = sPrimaryWINS
            SecondaryWINS = sSecondaryWINS
        End Sub
        Public iDHCP As Integer
        Public DeviceIP As String
        Public GateWay As String
        Public SubNetMask As String
        Public PrimaryDNS As String
        Public SecondaryDNS As String
        Public PrimaryWINS As String
        Public SecondaryWINS As String
    End Structure

    Private m_pSmartConfig As SmartX.SmartConfigs
    Private m_NetworkDeviceType As NetDeviceType
    Private m_strFailMessage As String
    Private m_iIPAllocationRetryMax As Integer
    Private m_iIPAllocationRetryCount As Integer
    Private m_iServerConnectRetryMax As Integer
    Private m_iServerConnectRetryCount As Integer
    Private m_strAPAddress As String
    Private m_strAPName As String
    Private m_strAPPassword As String
    Private m_IPInfos As NetIPSetInfos
    Private m_CompletionTimer As System.Windows.Forms.Timer
    Private m_MainTimer As System.Windows.Forms.Timer
    Private m_strMACAddressFilePath As String

    Public Delegate Sub FailHandler(ByVal fStatus As FailStatus)
    Public Event On_Failed As FailHandler
    Public Delegate Sub CompletionHandler(ByRef bServerConnect As Boolean)
    Public Event On_Completion As CompletionHandler

    Public Sub New()
        m_pSmartConfig = Nothing
        m_NetworkDeviceType = NetDeviceType.WiredLAN
        m_strFailMessage = ""
        m_iIPAllocationRetryMax = 20
        m_iIPAllocationRetryCount = 0
        m_iServerConnectRetryMax = 10
        m_iServerConnectRetryCount = 0
        m_strAPAddress = ""
        m_strAPName = ""
        m_strAPPassword = ""
        m_CompletionTimer = New System.Windows.Forms.Timer()
        AddHandler m_CompletionTimer.Tick, AddressOf CompletionTimer_Tick
        m_MainTimer = New System.Windows.Forms.Timer()
        AddHandler m_MainTimer.Tick, AddressOf MainTimer_Tick
    End Sub

    Public WriteOnly Property pSmartConfig() As SmartX.SmartConfigs
        Set(ByVal value As SmartX.SmartConfigs)
            m_pSmartConfig = value
        End Set
    End Property

    Public WriteOnly Property NetworkDeviceType() As NetDeviceType
        Set(ByVal value As NetDeviceType)
            m_NetworkDeviceType = value
        End Set
    End Property

    Public WriteOnly Property FailedMessage() As String
        Set(ByVal value As String)
            m_strFailMessage = value
        End Set
    End Property

    Public WriteOnly Property IPAllocationRetryMax() As Integer
        Set(ByVal value As Integer)
            m_iIPAllocationRetryMax = value
        End Set
    End Property

    Public ReadOnly Property GetIPAllocationRetryCnt() As Integer
        Get
            Return m_iIPAllocationRetryCount
        End Get
    End Property

    Public ReadOnly Property GetServerConnectRetryCnt() As Integer
        Get
            Return m_iServerConnectRetryCount
        End Get
    End Property

    Public WriteOnly Property ServerConnectRetryMax() As Integer
        Set(ByVal value As Integer)
            m_iServerConnectRetryMax = value
        End Set
    End Property

    Public Sub SetIPSettings(ByVal IPInfos As NetIPSetInfos)
        m_IPInfos = IPInfos
    End Sub

    Public Sub SetAPInfos(ByVal strAPAddress As String, ByVal strAPName As String, ByVal strAPPassword As String)
        m_strAPAddress = strAPAddress
        m_strAPName = strAPName
        m_strAPPassword = strAPPassword
    End Sub

    Public WriteOnly Property MACAddressFilePathName() As String
        Set(ByVal value As String)
            m_strMACAddressFilePath = value
        End Set
    End Property

    Public Sub Start()
        IPInfosChange(m_IPInfos)
        m_MainTimer.Interval = 700
        m_MainTimer.Enabled = True
    End Sub

    Public Sub [Stop]()
        m_MainTimer.Enabled = False
        m_CompletionTimer.Enabled = False
        m_iIPAllocationRetryCount = 0
        m_iServerConnectRetryCount = 0
    End Sub

    '// 유무선 네트워크 타입에 따른 IP 할당 시도 및 할당 여부에 따라 IP 할당 재시도와 Server 연결을 진행 하는 함수
    Private Function Network_Connect() As Boolean

        '// Server 연결을 진행하기 위한 TImer Interval
        m_CompletionTimer.Interval = 300

        '// 유선,무선 랜 타입에 따른 IP 할당을 시도
        Select Case m_NetworkDeviceType
            '// 유선랜 사용
            Case NetDeviceType.WiredLAN
                '// 유선랜으로 네트워크에 연결 했을때 IP를 할당 받아 네트워크 통신이 가능한지 확인 
                If m_pSmartConfig.IPSettings.IsLocalNetworkAvailable() = True Then
                    m_CompletionTimer.Enabled = True
                    Return True
                End If
                '// 구형 무선랜 사용
            Case NetDeviceType.WirelessLAN_RT2501
                m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501

                If m_strAPPassword = "" Then
                    '// 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                    If m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501, m_strAPName, m_strAPAddress, 1, 1, 10) = SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED Then
                        m_CompletionTimer.Enabled = True
                        Return True
                    End If
                Else
                    '// 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                    If m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501, m_strAPName, m_strAPPassword, m_strAPAddress, 1, 1, 10) = SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED Then
                        m_CompletionTimer.Enabled = True
                        Return True
                    End If
                End If
                '// 신형 무선랜 사용
            Case NetDeviceType.WirelessLAN_RT2870
                m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870

                If m_strAPPassword = "" Then
                    '// 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                    If m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870, m_strAPName, m_strAPAddress, 1, 1, 10) = SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED Then
                        m_CompletionTimer.Enabled = True
                        Return True
                    End If
                Else
                    '// 무선랜과 AP의 연결을 시도하여 결과값을 리턴 받아 AP와 연결 재시도 처리 진행
                    If m_pSmartConfig.USBWirelessIPSettings.SetUSBWirelessLANConfig_CheckRetry(SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870, m_strAPName, m_strAPPassword, m_strAPAddress, 1, 1, 10) = SmartX.CWUSBIPSetting.APCONNECTSTATUS.CONNECTED Then
                        m_CompletionTimer.Enabled = True
                        Return True
                    End If
                End If
        End Select

        '// IP 할당 시도 최대 횟수까지 재시도
        If System.Threading.Interlocked.Increment(m_iIPAllocationRetryCount) < m_iIPAllocationRetryMax Then
            '// IP 할당을 진행하기 위한 Timer 시작
            m_MainTimer.Enabled = True
        Else
            '// IP 할당 실패에 따른 처리
            If m_strFailMessage <> "" Then

                SmartX.SmartMessageBox.Size(515, 190)
                SmartX.SmartMessageBox.Show(m_strFailMessage, "IP 할당 실패", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Hand, System.Windows.Forms.MessageBoxDefaultButton.Button1)
            End If
            RaiseEvent On_Failed(FailStatus.IP_Allocation_Failed)
        End If

        Return False
    End Function

    '// 유무선 네트워크 타입에 따라 IP, GateWay, SubNetMask, DNS, WINS 주소를 설정하는 함수
    Private Sub IPInfosChange(ByVal netIPInfos As NetIPSetInfos)
        Dim bUpdateFlag As Boolean = False

        Select Case m_NetworkDeviceType
            '// 유선랜 사용
            Case NetDeviceType.WiredLAN
                '// 동적 IP 사용
                If m_pSmartConfig.IPSettings.DHCPEnable <> netIPInfos.iDHCP Then
                    m_pSmartConfig.IPSettings.DHCPEnable = netIPInfos.iDHCP
                End If

                '// 고정 IP 사용
                If netIPInfos.iDHCP = 0 Then

                    If (netIPInfos.DeviceIP IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.DeviceIP <> netIPInfos.DeviceIP) Then
                        m_pSmartConfig.IPSettings.DeviceIP = netIPInfos.DeviceIP
                    End If

                    If (netIPInfos.GateWay IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.GateWay <> netIPInfos.GateWay) Then
                        m_pSmartConfig.IPSettings.GateWay = netIPInfos.GateWay
                    End If

                    If (netIPInfos.SubNetMask IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.SubNetMask <> netIPInfos.SubNetMask) Then
                        m_pSmartConfig.IPSettings.SubNetMask = netIPInfos.SubNetMask
                    End If

                    If (netIPInfos.PrimaryDNS IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.PrimaryDNS <> netIPInfos.PrimaryDNS) Then
                        m_pSmartConfig.IPSettings.PrimaryDNS = netIPInfos.PrimaryDNS
                    End If

                    If (netIPInfos.SecondaryDNS IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.SecondaryDNS <> netIPInfos.SecondaryDNS) Then
                        m_pSmartConfig.IPSettings.SecondaryDNS = netIPInfos.SecondaryDNS
                    End If

                    If (netIPInfos.PrimaryWINS IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.PrimaryWINS <> netIPInfos.PrimaryWINS) Then
                        m_pSmartConfig.IPSettings.PrimaryWINS = netIPInfos.PrimaryWINS
                    End If

                    If (netIPInfos.SecondaryWINS IsNot Nothing) AndAlso (m_pSmartConfig.IPSettings.SecondaryWINS <> netIPInfos.SecondaryWINS) Then
                        m_pSmartConfig.IPSettings.SecondaryWINS = netIPInfos.SecondaryWINS
                    End If
                End If
                '// IP 주소 설정값을 레지스트리에 저장
                m_pSmartConfig.IPSettings.Save()
                '// 설정값을 적용
                m_pSmartConfig.IPSettings.SetApply()

                '// 무선랜 사용
            Case NetDeviceType.WirelessLAN_RT2501, NetDeviceType.WirelessLAN_RT2870

                If m_NetworkDeviceType = NetDeviceType.WirelessLAN_RT2501 Then
                    m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2501
                Else
                    m_pSmartConfig.USBWirelessIPSettings.USBWirelessType = SmartX.CWUSBIPSetting.USBWLANTYPE.RT2870
                End If
                '// 동적 IP 사용
                If m_pSmartConfig.USBWirelessIPSettings.DHCPEnable <> netIPInfos.iDHCP Then
                    m_pSmartConfig.USBWirelessIPSettings.DHCPEnable = netIPInfos.iDHCP
                    bUpdateFlag = True
                End If

                '// 고정 IP 사용
                If netIPInfos.iDHCP = 0 Then

                    If (netIPInfos.DeviceIP IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.DeviceIP <> netIPInfos.DeviceIP) Then
                        m_pSmartConfig.USBWirelessIPSettings.DeviceIP = netIPInfos.DeviceIP
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.GateWay IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.GateWay <> netIPInfos.GateWay) Then
                        m_pSmartConfig.USBWirelessIPSettings.GateWay = netIPInfos.GateWay
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.SubNetMask IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.SubNetMask <> netIPInfos.SubNetMask) Then
                        m_pSmartConfig.USBWirelessIPSettings.SubNetMask = netIPInfos.SubNetMask
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.PrimaryDNS IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.PrimaryDNS <> netIPInfos.PrimaryDNS) Then
                        m_pSmartConfig.USBWirelessIPSettings.PrimaryDNS = netIPInfos.PrimaryDNS
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.SecondaryDNS IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.SecondaryDNS <> netIPInfos.SecondaryDNS) Then
                        m_pSmartConfig.USBWirelessIPSettings.SecondaryDNS = netIPInfos.SecondaryDNS
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.PrimaryWINS IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.PrimaryWINS <> netIPInfos.PrimaryWINS) Then
                        m_pSmartConfig.USBWirelessIPSettings.PrimaryWINS = netIPInfos.PrimaryWINS
                        bUpdateFlag = True
                    End If

                    If (netIPInfos.SecondaryWINS IsNot Nothing) AndAlso (m_pSmartConfig.USBWirelessIPSettings.SecondaryWINS <> netIPInfos.SecondaryWINS) Then
                        m_pSmartConfig.USBWirelessIPSettings.SecondaryWINS = netIPInfos.SecondaryWINS
                        bUpdateFlag = True
                    End If
                End If

                If bUpdateFlag = True Then
                    '// IP 주소 설정값을 레지스트리에 저장
                    m_pSmartConfig.USBWirelessIPSettings.Save()
                    '// 설정값을 적용
                    m_pSmartConfig.USBWirelessIPSettings.SetApply()
                End If
        End Select
    End Sub

    '// MAC Adress 설정을 위한 함수
    Public Function MacAddress_Setting() As Boolean
        Dim smartFile1 As SmartX.SmartFile = New SmartX.SmartFile()

        '// txt 파일에서 읽어온 MAC Address를 저장할 변수
        Dim strRawMacaddress As String
        '// MAC Address를 ':' 로 분할하여 각각 나눈 값을 저장할 배열
        Dim strMacAddress As String()
        '// MAC Address 설정을 위해 분할된 값을 하나의 MAC Address로 저장할 배열
        Dim bSetMacAddress As Byte() = New Byte(5) {}

        '// MAC Address 설정 값을 가지고 있는 txt 파일이 존재 하지 않는경우
        If (m_strMACAddressFilePath = "") OrElse (System.IO.File.Exists(m_strMACAddressFilePath) = False) Then
            Return False
        End If

        '// 읽어올 MAC Address txt 파일의 경로 설정 및 File Open
        smartFile1.FilePathName = m_strMACAddressFilePath
        smartFile1.Open()
        '// txt 파일의 MAC Address 설정 값을 문자열로 읽기
        strRawMacaddress = smartFile1.StringType.Read()
        smartFile1.Close()

        '// MAC Address 값의 길이가 17 자리가 아닌 경우
        If strRawMacaddress.Length <> 17 Then
            Return False
        End If

        strMacAddress = strRawMacaddress.Split(":"c)

        '// 분할하여 나눈 MAC Address 값의 길이가 6 자리가 아닌 경우
        If strMacAddress.Length <> 6 Then
            Return False
        End If

        Try
            '// 분할하여 나눈 MAC Address 길이 만큼 반복
            For i As Integer = 0 To strMacAddress.Length - 1
                bSetMacAddress(i) = CByte(Convert.ToInt16(strMacAddress(i).ToString(), 16))
            Next

        Catch __unusedFormatException1__ As FormatException
            Return False
        End Try

        Dim bMacAddress As Byte() = m_pSmartConfig.IPSettings.GetMACAddress()

        '// 설정 되어 있는 MAC Address와 파일로 읽어온 MAC Address 값이 같지 않은 경우
        If Enumerable.SequenceEqual(bSetMacAddress, bMacAddress) = False Then
            '// 파일에서 읽어온 MAC Address값으로 설정
            m_pSmartConfig.IPSettings.SetMACAddress(bSetMacAddress)
            '// MAC Address 적용을 위한 IEC-Series 재부팅 여부 확인
            Dim eRebootResult As DialogResult = SmartX.SmartMessageBox.Show("MAC Address Setting이 완료되었습니다." & vbCrLf & "재부팅을 진행하시겠습니까?", "재부팅 여부 확인", MessageBoxButtons.YesNo, MessageBoxIcon.Question)

            If eRebootResult = DialogResult.Yes Then
                SmartX.SmartConfigs.ReBoot()
            End If
        End If

        Return True
    End Function

    '// IP 할당을 시도하기 위한 이벤트
    Private Sub MainTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        m_MainTimer.Enabled = False
        Network_Connect()
    End Sub

    '// IP 할당 여부에 따라 Server 연결을 시도하기 위한 이벤트
    Private Sub CompletionTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)

        m_CompletionTimer.Enabled = False

        '// Server 연결 성공, 실패 여부를 확인하기 위한 변수
        Dim bServerConnectFlag As Boolean = False

        If On_CompletionEvent IsNot Nothing Then
            '// Server 연결에 성공, 실패 여부 확인
            RaiseEvent On_Completion(bServerConnectFlag)

            '// Server 연결에 실패한 경우
            If bServerConnectFlag = False Then

                '// Server 연결 시도 최대 횟수 까지 재시도
                If System.Threading.Interlocked.Increment(m_iServerConnectRetryCount) <= m_iServerConnectRetryMax Then
                    m_CompletionTimer.Enabled = True
                Else

                    If On_FailedEvent IsNot Nothing Then
                        RaiseEvent On_Failed(FailStatus.ServerConnect_Failed)
                    End If
                End If
            End If
        End If
    End Sub
End Class
